{
*********************************************************************************
Title: Header Comment Expert
Version: 1.2 (32-Bit) for Delphi v2
Date/Time: 04/22/97 @ 07:49:19 AM
Copyright: 1996-1997 Computech Services
Comment: Project Source for Header Comment Expert (CMTEXPT.DPR)
         By: David M. Gentils
*********************************************************************************
}
unit cmtexptu;

interface

uses Windows, Messages, Classes, Graphics, Forms, Controls, Menus,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls, SysUtils, Clipbrd,
  frmPreview, StandInc, AboutDlg, Nag, ExptIntf, ToolIntf, EditIntf,
  constnts, frmDate, Valid, helpdefs;

type
    TBorderChars = (Asterisk, Minus, Plus, Tilde);

type
  TMainForm = class(TForm)
    Toolbar: TPanel;
    OpenBtn: TSpeedButton;
    SaveBtn: TSpeedButton;
    NewBtn: TSpeedButton;
    UndoBtn: TSpeedButton;
    CutBtn: TSpeedButton;
    CopyBtn: TSpeedButton;
    PasteBtn: TSpeedButton;
    Label1: TLabel;
    txtHeader: TEdit;
    cboBorder: TComboBox;
    Memo1: TMemo;
    Label2: TLabel;
    txtCopyright: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    txtDateTime: TEdit;
    cmdInsertDateTime: TButton;
    Label5: TLabel;
    txtVersion: TEdit;
    Label6: TLabel;
    SendBtn: TSpeedButton;
    PreviewBtn: TSpeedButton;
    SaveAsBtn: TSpeedButton;
    SpeedButton2: TSpeedButton;
    txtComment1: TEdit;
    Label7: TLabel;
    txtComment2: TEdit;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    Image1: TImage;
    MainMenu1: TMainMenu;
    mnuFile: TMenuItem;
    mnuFileNew: TMenuItem;
    mnuFileOpen: TMenuItem;
    mnuFileSave: TMenuItem;
    mnuSaveAs: TMenuItem;
    mnuEdit: TMenuItem;
    mnuUndo: TMenuItem;
    N1: TMenuItem;
    mnuCut: TMenuItem;
    mnuEditCopy: TMenuItem;
    mnuEditPaste: TMenuItem;
    N2: TMenuItem;
    mnuFileExit: TMenuItem;
    N4: TMenuItem;
    mnuEditComment: TMenuItem;
    mnuCommentPreview: TMenuItem;
    mnuSend: TMenuItem;
    cboMods: TComboBox;
    Label8: TLabel;
    BrowseBtn: TButton;
    Validator: TValidator;
    NagScreen: TNagScreen;
    HelpMenu: TMenuItem;
    mnuRegister: TMenuItem;
    Bevel1: TBevel;
    HelpContents: TMenuItem;
    N3: TMenuItem;
    HelpAbout: TMenuItem;
    N5: TMenuItem;
    procedure EditChange(Sender: TObject);
    procedure SaveItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cboBorderChange(Sender: TObject);
    procedure cmdSendClick(Sender: TObject);
    procedure cmdInsertDateTimeClick(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure Undo1Click(Sender: TObject);
    procedure Cut1Click(Sender: TObject);
    procedure Copy1Click(Sender: TObject);
    procedure Paste1Click(Sender: TObject);
    procedure PreviewBtnClick(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure mnuFileExitClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BrowseBtnClick(Sender: TObject);
    procedure ValidatorVldFailed(Sender: TObject);
    procedure ValidatorVldOk(Sender: TObject);
    procedure mnuRegisterClick(Sender: TObject);
    procedure HelpAboutClick(Sender: TObject);
    procedure HelpContentsClick(Sender: TObject);
  private
    { Private declarations }
    ModsList: TStringList;
    BorderChar: TBorderChars;
    CurrentFile: string;
    StringList: TStringList;
    ValidationOK: boolean;
    procedure BuildBorder(StringList: TStringList);
    procedure SaveFile(const Filename: string);
    procedure ClearDisplay;
    procedure EnableUI(Enable: boolean);
  public
    { Public declarations }
    function GetBorderChar: char;
  end;

TCommentExpert = class(TIExpert)
  protected
    MenuItem: TIMenuItemIntf;
  public
    constructor Create;
    destructor Destroy; override;
    function GetStyle: TExpertStyle; override;
    function GetIDString: string; override;
    function GetName: string; override;
    procedure MenuItemClick(Sender: TIMenuItemIntf);
  end;

var
  MainForm: TMainForm;
  Day, Month, Year: integer;
  Hour, Minute, Second, AMPM: string;

implementation

{$R *.DFM}

constructor TCommentExpert.Create;
var
  mainbar: TIMainMenuIntf;
  mainmenu, prevmenu, parentmenu: TIMenuItemIntf;
begin

  inherited Create;

  { Insert the menu item }
  mainbar := ToolServices.GetMainMenu;
  try
    mainmenu := mainbar.GetMenuItems;
    try
      prevmenu := mainbar.FindMenuItem('ViewObjInspItem');
      try
        parentmenu := prevmenu.GetParent;
        try
          menuitem := parentmenu.InsertItem(prevmenu.GetIndex,
            'Header Comment Expert','ViewCommentExpertItem', '', 0, 0, 0,
            [mfVisible, mfEnabled], MenuItemClick);
        finally
          parentmenu.free;
        end;
      finally
        prevmenu.free;
      end;
    finally
      mainmenu.free;
    end;
  finally
    mainbar.free;
  end;
end;

destructor TCommentExpert.Destroy;
begin
  menuitem.DestroyMenuItem;
  menuitem.Free;
  inherited Destroy;
end;

procedure TCommentExpert.MenuItemClick(Sender: TIMenuItemIntf);
begin

  if Sender <> menuitem then
    Exit;

  { Our menu item was clicked, so open up the
    form and start the ball rolling. }

  { Create the form if necessary }
  if not Assigned(MainForm) then
    MainForm := TMainForm.Create(Application);

  { Show the form and set the focus. Note that since I'm
    filling the unit list here, choosing this menu item
    while the form is already on the screen will update
    the dropdown unit list. If the reader adds notifications,
    this probably won't be necessary. }
  MainForm.Show;
  MainForm.SetFocus;
end;

function TCommentExpert.GetStyle: TExpertStyle;
begin
  Result := esAddIn;
end;

function TCommentExpert.GetIDString: String;
begin
  Result := 'Delphi Expert.Header Comment Expert';
end;

function TCommentExpert.GetName: String;
begin
  Result := 'Header Comment Expert';
end;

procedure TMainForm.EditChange(Sender: TObject);
begin
     EnableUI(True);
end;

function TMainForm.GetBorderChar: char;
var
   bChar: char;
begin
     case BorderChar of
          Asterisk: bChar := '*';
          Plus: bChar := '+';
          Minus: bChar := '-';
          Tilde: bChar := '~';
     end;
     Result := bChar;
end;

procedure TMainForm.BuildBorder(StringList: TStringList);
var
   i: byte;
   Temp: string;
   bChar: char;
begin
     bChar := GetBorderChar;
     for i := 0 to 80 do
         Temp := Temp + bChar;
     StringList.Add(Temp);
end;

procedure TMainForm.SaveFile(const Filename: string);
var
   S: TextFile;
begin
     Screen.Cursor := crHourglass;
     AssignFile(S, Filename);
     Rewrite(S);
     WriteLn(S, txtHeader.Text);
     WriteLn(S, txtCopyright.Text);
     WriteLn(S, txtDateTime.Text);
     WriteLn(S, txtVersion.Text);
     WriteLn(S, txtComment1.Text);
     WriteLn(S, txtComment2.Text);
     CloseFile(S);
     Screen.Cursor := crDefault;
end;

procedure TMainForm.ClearDisplay;
begin
     txtHeader.Text := '';
     txtCopyright.Text := '';
     txtDateTime.Text := '';
     txtVersion.Text := '';
     txtComment1.Text := '';
     txtComment2.Text := '';
     BorderChar := Asterisk;
     cboBorder.ItemIndex := Ord(BorderChar);
     Caption := IDS_PROGRAM_TITLE;
     txtVersion.SetFocus;
     Memo1.Text := '';
     Memo1.Visible := False;
end;

procedure TMainForm.EnableUI(Enable: boolean);
begin
     PreviewBtn.Enabled := Enable;
     mnuEditComment.Enabled := Enable;
     SendBtn.Enabled := Enable;
     SaveBtn.Enabled := Enable;
     mnuFileSave.Enabled := Enable;
     SaveAsBtn.Enabled := Enable;
     mnuSaveAs.Enabled := Enable;
end;

procedure TMainForm.SaveItemClick(Sender: TObject);
begin
     if CurrentFile <> '' then
        SaveFile(CurrentFile)
     else
         SaveAs1Click(Sender);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.HelpFile := ExtractFilePath(Application.EXEName) + 'Cmtexpt.hlp';
  Icon := Image1.Picture.Icon;
  Caption := IDS_PROGRAM_TITLE;
  BorderChar := Asterisk;
  cboBorder.ItemIndex := Ord(BorderChar);
  txtHeader.Text := '';
  txtCopyright.Text := '';
  txtDateTime.Text := '';
  txtVersion.Text := '';
  txtComment1.Text := '';
  txtComment2.Text := '';
  CurrentFile := '';
  EnableUi(False);
  ModsList := TStringList.Create;
  if not Validator.IsValidated then
  begin
       ValidationOK := False;
       NagScreen.Comments := IDS_NAG_COMMENT;
       NagScreen.Execute;
       mnuRegister.Enabled := True;
  end
  else
  begin
       ValidationOK := True;
       mnuRegister.Enabled := False;
  end;
end;

procedure TMainForm.cboBorderChange(Sender: TObject);
begin
     case cboBorder.ItemIndex of
          0: BorderChar := Asterisk;
          1: BorderChar := Minus;
          2: BorderChar := Plus;
          3: BorderChar := Tilde;
     end;
     EnableUI(True);
end;

procedure TMainForm.cmdSendClick(Sender: TObject);
var
   unitmod: TIModuleInterface;
   editint: TIEditorInterface;
   editwrite: TIEditWriter;
   StringList: TStringList;
   modname: string;
begin
     if not ValidationOK then
     begin
          MessageBeep(0);
          NagScreen.TimeDelay := 3;
          NagScreen.Comments := IDS_NAG_COMMENT;
          NagScreen.Execute;
          mnuRegister.Enabled := True;
     end;
     if not Assigned(ToolServices) then Exit;
     if ToolServices.GetProjectName = '' then
     begin
        MessageBeep(0);
        Application.MessageBox(PChar(IDS_NOPROJECT), PChar(IDS_PROGRAM_TITLE),
                                                     MB_OK + MB_ICONEXCLAMATION);
        Exit;
     end;
     MessageBeep(0);
     if Application.MessageBox(PChar(IDS_SENDMSG_PROMPT), 'Send to Delphi',
                               MB_OKCANCEL + MB_ICONQUESTION) = IDCANCEL then exit;
     StringList := TStringList.Create;
     StringList.Add('{');
     BuildBorder(StringList);
     StringList.Add('Title: ' + txtHeader.Text);
     StringList.Add('Version: ' + txtVersion.Text);
     StringList.Add('Date/Time: ' + txtDateTime.Text);
     StringList.Add('Copyright: ' + txtCopyright.Text);
     StringList.Add('Comment: ' + txtComment1.Text);
     StringList.Add('         ' + txtComment2.Text);
     BuildBorder(StringList);
     StringList.Add('}');
     Memo1.Lines.Assign(StringList);
     {ToolServices.OpenFile(ToolServices.GetCurrentFile);}
     modname := ModsList.Strings[cboMods.ItemIndex];
     if Pos('None', modname) <> 0 then
     begin
        ToolServices.OpenFile(ToolServices.GetProjectName);
        unitmod := ToolServices.GetModuleInterface(ToolServices.GetProjectName);
     end
     else
     begin
         ToolServices.OpenFile(ModsList.Strings[cboMods.ItemIndex]);
         unitmod := ToolServices.GetModuleInterface(ModsList.Strings[cboMods.ItemIndex]);
     end;
     editint := unitmod.GetEditorInterface;
     editwrite := editint.CreateWriter;
     editwrite.Insert(PChar(Memo1.Text));
     editwrite.Free;
     editint.Free;
     unitmod.Free;
     StringList.Free;
     Close;
end;

procedure TMainForm.cmdInsertDateTimeClick(Sender: TObject);
begin
     txtDateTime.Text := FormatDateTime('mm/dd/yy', Date) + ' @ ' +
                      FormatDateTime('hh:mm:ss AM/PM', Time);
     EnableUI(True);
end;

procedure TMainForm.SaveAs1Click(Sender: TObject);
begin
     SaveDialog.Filter := IDS_SAVEDIALOG_FILTER;
     SaveDialog.Title := IDS_SAVEDIALOG_TITLE;
     SaveDialog.FileName := 'untitled.cmt';
     if SaveDialog.Execute then
     begin
          CurrentFile := SaveDialog.Filename;
          SaveFile(CurrentFile);
          Caption := IDS_PROGRAM_TITLE + ' - ' + CurrentFile;
     end;
end;

procedure TMainForm.Open1Click(Sender: TObject);
var
   S: TextFile;
   Temp: string;
begin
     OpenDialog.Filter := IDS_OPENDIALOG_FILTER;
     OpenDialog.Title := IDS_OPENDIALOG_TITLE;
     OpenDialog.FileName := '*.cmt';
     OpenDialog.InitialDir := ExtractFilePath(ToolServices.GetProjectName);
     if OpenDialog.Execute then
     begin
          CurrentFile := OpenDialog.Filename;
          Screen.Cursor := crHourglass;
          ClearDisplay;
          AssignFile(S, CurrentFile);
          try
             Reset(S);
             ReadLn(S, Temp);
             txtHeader.Text := Temp;
             ReadLn(S, Temp);
             txtCopyright.Text := Temp;
             ReadLn(S, Temp);
             txtDateTime.Text := Temp;
             ReadLn(S, Temp);
             txtVersion.Text := Temp;
             ReadLn(S, Temp);
             txtComment1.Text := Temp;
             ReadLn(S, Temp);
             txtComment2.Text := Temp;
          finally
                 CloseFile(S);
          end;
          Caption := IDS_PROGRAM_TITLE + ' - ' + CurrentFile;
          Screen.Cursor := crDefault;
     end;
end;

procedure TMainForm.New1Click(Sender: TObject);
begin
     ClearDisplay;
     CurrentFile := '';
end;

procedure TMainForm.Undo1Click(Sender: TObject);
begin
     SendMessage(ActiveControl.Handle, WM_UNDO, 0, 0);
end;

procedure TMainForm.Cut1Click(Sender: TObject);
begin
     SendMessage(ActiveControl.Handle, WM_CUT, 0, 0);
end;

procedure TMainForm.Copy1Click(Sender: TObject);
begin
     SendMessage(ActiveControl.Handle, WM_COPY, 0, 0);
end;

procedure TMainForm.Paste1Click(Sender: TObject);
begin
     if Clipboard.HasFormat(CF_TEXT) then
        SendMessage(ActiveControl.Handle, WM_PASTE, 0, 0)
     else
         MessageBeep(0);
         Application.MessageBox(PChar(IDS_ERR_CLIPFORMAT), 'Paste Error',
                                MB_OK + MB_ICONSTOP);
end;

procedure TMainForm.PreviewBtnClick(Sender: TObject);
begin
     PreviewDlg := TPreviewDlg.Create(Application);
     try
        PreviewDlg.ShowModal;
        Memo1.Visible := False;
     finally
            PreviewDlg.Free;
     end;
end;

procedure TMainForm.SpeedButton2Click(Sender: TObject);
var
   AboutDialog: TAboutDialog;
begin
     AboutDialog := TAboutDialog.Create(Application);
     try
        with AboutDialog do
        begin
             Caption := IDS_ABOUT_TITLE;
             ProductName.Caption := IDS_PROGRAM_TITLE;
             if ValidationOK then
                Version.Caption := 'Version: ' + IDS_VERSION
             else
                 Version.Caption := 'Version: ' + IDS_SWARE_VERSION;
             CopyRight.Caption := IDS_ABOUT_OTHER;
             Comments.Caption := IDS_NAG_AUTHOR;
             ProgramIcon.Picture.Graphic := Image1.Picture.Icon;
             ShowModal;
        end;
     finally
            AboutDialog.Free;
     end;
end;

procedure TMainForm.mnuFileExitClick(Sender: TObject);
begin
     Close;
end;

procedure TMainForm.FormShow(Sender: TObject);
var
   i: integer;
   str, str2: string;
begin
     if ToolServices.GetProjectName = '' then
     begin
        MessageBeep(0);
        Application.MessageBox(PChar(IDS_NOPROJECT), PChar(IDS_PROGRAM_TITLE),
                                                     MB_OK + MB_ICONEXCLAMATION);
     end;
     cboMods.Items.Clear;
     ModsList.Clear;
     i := ToolServices.GetUnitCount;
     if i = 0 then
     begin
          cboMods.Items.Add('None');
          cboMods.ItemIndex := 0;
          ModsList.Add('None');
          Exit;
     end;
     for i := 0 to i - 1 do
     begin
          str := ToolServices.GetUnitName(i);
          if Pos('.PAS', UpperCase(str)) <> 0 then
          begin
               ModsList.Add(str);
               str2 := UpperCase(ExtractFileName(str));
               cboMods.Items.Add(LeftStr(str2, Length(str2) - 4));
          end
          else
          begin
               if Pos('.DPR', UpperCase(str)) <> 0 then
               begin
                    ModsList.Add(str);
                    str2 := UpperCase(ExtractFileName(str));
                    cboMods.Items.Add(LeftStr(str2, Length(str2) - 4));
               end;
          end;
     end;
     str2 := UpperCase(ExtractFileName(ToolServices.GetCurrentFile));
     cboMods.ItemIndex := cboMods.Items.IndexOf(LeftStr(str2, Length(str2) - 4));
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
     ModsList.Free;
     Application.HelpCommand(HELP_QUIT, 0);
end;

procedure TMainForm.BrowseBtnClick(Sender: TObject);
var
   YearTemp: string;
begin
     SelectDateDlg := TSelectDateDlg.Create(Application);
     try
        SelectDateDlg.ShowModal;
        if SelectDateDlg.ModalResult = mrOK then
        begin
             YearTemp := IntToStr(Year);
             txtDateTime.Text := IntToStr(Month) + '/' + IntToStr(Day) + '/' +
                              RightStr(YearTemp, 2) + ' ' + '@ ' + Hour + ':' + Minute +
                              ':' + Second + ' ' + AMPM;
        end;
     finally
            SelectDateDlg.Free;
     end;
end;

procedure TMainForm.ValidatorVldFailed(Sender: TObject);
begin
     MessageBeep(MB_ICONSTOP);
     Application.MessageBox('Validation Failed, this program is not registered. ' +
                                        'The registration information you have entered ' +
                                        'is incorrect! Try Again.', 'Registration Invalid', MB_OK +
                                        MB_ICONSTOP);
     ValidationOK := False;
end;

procedure TMainForm.ValidatorVldOk(Sender: TObject);
begin
     MessageBeep(0);
     Application.MessageBox('Validation Accepted, this program is now registered! ' +
                                        'Thank You very much.', 'Registration Validated',
                                        MB_OK + MB_ICONEXCLAMATION);
     mnuRegister.Enabled := False;
     ValidationOK := True;
end;

procedure TMainForm.mnuRegisterClick(Sender: TObject);
begin
     if ValidationOk then
     begin
          Validator.Devalidate;
          ValidationOk := false;
     end
     else
     begin
          Validator.Validate;
          ValidationOk := Validator.IsValidated;
     end;
end;

procedure TMainForm.HelpAboutClick(Sender: TObject);
begin
     SpeedButton2Click(Sender);
end;

procedure TMainForm.HelpContentsClick(Sender: TObject);
begin
     Application.HelpCommand(HELP_FINDER, 0);
end;

end.

